@@ -2,6 +2,7 @@
 package ExtUtils::F77;
 
 use Config;
+use File::Spec;
 
 =head1 NAME
 
@@ -20,7 +21,7 @@ A simple self-documenting Perl database of knowledge/code
 for figuring out how to link for various combinations of OS and
 compiler is embedded in the modules Perl code. Please help 
 save the world by sending database entries for
-your system to kgb@aaoepp.aao.gov.au
+your system to karl_pgplot@mac.com
 
 The library list which the module returns 
 can be explicitly overridden by setting the environment 
@@ -32,7 +33,9 @@ variable F77LIBS, e.g.
 
 =cut
 
-$VERSION = "1.11";
+$VERSION = "1.16";
+
+warn "\nExtUtils::F77: Version $VERSION\n";
 
 # Database starts here. Basically we have a large hash specifying
 # entries for each os/compiler combination. Entries can be code refs
@@ -43,6 +46,8 @@ $VERSION = "1.11";
 # hash keys. First key is usually the name of the architecture as
 # returned by Config (modulo ucfirst()).
 
+print "Loaded ExtUtils::F77 version $VERSION\n";
+
 %F77config=();
 
 ### SunOS (use this as a template for new entries) ###
@@ -78,12 +83,85 @@ $F77config{Sunos}{F77}{Cflags} = '-O';
 
 ### Solaris ###
 
-$F77config{Solaris}{F77}{Link} = sub {  
-       $dir = find_highest_SC("/opt/SUNWspro/SC*/lib");
-       return "" unless $dir; # Failure
-       print "$Pkg: Found Fortran latest version lib dir $dir\n";
-       return "-L$dir -lF77 -lM77 -lsunmath -lm";
-};
+$F77config{Solaris}{F77}{Link} = sub {
+     my $NSPATH;
+     my $dir;
+
+     #find the SUNWspro entry of nonstandard inst. in LD_LIBRARY_PATH
+     if ( defined $ENV{'LD_LIBRARY_PATH'} &&
+	  $ENV{'LD_LIBRARY_PATH'} =~ m{([^:]*SUNWspro[^/]*)} )
+     {
+       $NSPATH = $1;
+     }
+
+     elsif ( defined $ENV{'PATH'} ) {
+
+       foreach ( split (/:/,$ENV{PATH}) ) {
+	 if ( m{(.*SUNWspro[^/]*)} ) {
+	   $NSPATH = $1;
+	   last;
+	 }
+       }
+     }
+
+
+     if (defined $NSPATH) {
+
+       print "$Pkg: Found F77 path:--->$NSPATH\n";
+
+       $dir = find_highest_SC("$NSPATH/WS*/lib") ||
+              find_highest_SC("$NSPATH/SC*/lib") ||
+	      find_highest_SC("$NSPATH/sunwspro*/SUNWspro/SC*/lib");
+
+       # that failed.  try $NSPATH/lib. use glob to
+       # match for libF77.*, as libF77.a isn't there anymore?
+       unless ( $dir )
+       {
+	 print "$Pkg: Trying $NSPATH/lib\n";
+	 $dir = "$NSPATH/lib" if glob("$NSPATH/lib/libF77*");
+       }
+	      
+     } else {
+     }
+     return "" unless $dir; # Failure
+     print "$Pkg: Found Fortran latest version lib dir $dir\n";
+
+     my @libs;
+
+     # determine libraries. Forte 7 doesn't have F77 or M77
+     if ( qx/$F77config{Solaris}{F77}{Compiler} -V 2>&1/ 
+                      =~ /Forte Developer 7/ )
+     {
+       push @libs, qw/
+		      -lf77compat
+		      -lfui
+		      -lfai
+		      -lfai2
+		      -lfsumai
+		      -lfprodai
+		      -lfminlai
+		      -lfmaxlai
+		      -lfminvai
+		      -lfmaxvai
+		      -lfsu 
+		      -lsunmath
+		      -lm
+		      /;
+     }
+     else
+     {
+       push @libs, qw/
+		      -lF77
+		      -lM77
+		      -lsunmath
+		      -lm
+		      /;
+     }
+
+     join( ' ', "-L$dir", @libs );
+ };
+
+
 $F77config{Solaris}{F77}{Trail_} = 1;
 $F77config{Solaris}{F77}{Compiler} = 'f77';
 $F77config{Solaris}{F77}{Cflags} = '-O';
@@ -140,8 +218,44 @@ $F77config{Cygwin}{DEFAULT}	= 'G77';
 
 $F77config{Linux}{G77}     = $F77config{Generic}{G77};
 $F77config{Linux}{F2c}     = $F77config{Generic}{G77};
+#
+# database entry for the gfortran compiler
+#
+$F77config{Linux}{GFortran}{Link} = sub {
+    $dir = `gfortran -print-file-name=libgfortran.a`;
+    chomp $dir;
+    # Note that -print-file-name returns just the library name
+    # if it cant be found - make sure that we only accept the
+    # directory if it returns a proper path (or matches a /)
+
+    if( defined $dir ) {
+        $dir =~ s,/libgfortran.a$,,;
+    } else {
+        $dir = "/usr/local/lib";
+    }    
+    return( "-L$dir -L/usr/lib -lgfortran -lm" );
+};
+$F77config{Linux}{GFortran}{Trail_}   = 1;
+$F77config{Linux}{GFortran}{Compiler} = 'gfortran';
+$F77config{Linux}{GFortran}{Cflags}   = '-O';
 $F77config{Linux}{DEFAULT} = 'G77';
 
+# check for 'non g77' DEFAULT compilers on Linux:
+if (ucfirst($Config{'osname'}) eq "Linux") 
+{
+  my $default;
+
+  # check for gfortran compiler:
+  $default = find_in_path('gfortran');
+
+  if( $default =~ /gfortran/ )
+  {
+     $F77config{Linux}{DEFAULT} = 'GFortran'; 
+  } 
+
+  # code for ifort, g95 etc........
+}
+
 ### DEC OSF/1 ###
 
 $F77config{Dec_osf}{F77}{Link}   = "-L/usr/lib -lUfor -lfor -lFutil -lm -lots -lc";
@@ -168,6 +282,7 @@ $F77config{Hpux}{DEFAULT}     = 'F77';
 # in the ExtUtils::F77 package. It adds the -64 flag and it is supposed to
 # provide the same functionality as the old code for a non -64 abi. 
 
+if (ucfirst($Config{'osname'}) eq "Irix")
 {
   my ($cflags,$mips,$default_abi,$abi,$mips_dir,$libs);
   $cflags = $Config{cc};
@@ -177,7 +292,8 @@ $F77config{Hpux}{DEFAULT}     = 'F77';
   $default_abi = $Config{osvers} >= 6.4 ? "-n32" : "-o32";
  GET_ABI:
   {
-    $abi = "-o32",last GET_ABI if $cflags =~ /-o32/;
+    # -32 seems to be synonymous for -o32 (CS)
+    $abi = "-o32",last GET_ABI if $cflags =~ /-o?32/;
     $abi = "-n32",last GET_ABI if $cflags =~ /-n32/;
     $abi = "-64",last GET_ABI if $cflags =~ /-64/;
     $abi = $default_abi;
@@ -185,12 +301,12 @@ $F77config{Hpux}{DEFAULT}     = 'F77';
   if ( $abi eq "-64" ){
     $libs = ( (-r "/usr/lib64/$mips_dir") && (-d _) && (-x _) ) ?
 	"-L/usr/lib64/$mips_dir" : "";
-    $libs .=  " -L/usr/lib64 -lftn -lm";
+    $libs .=  " -L/usr/lib64 -lfortran -lm";
   }
   if ( $abi eq "-n32" ){
     $libs = ( (-r "/usr/lib32/$mips_dir") && (-d _) && (-x _) ) ?
 	"-L/usr/lib32/$mips_dir" : "";
-    $libs .=  " -L/usr/lib32 -lftn -lm";
+    $libs .=  " -L/usr/lib32 -lfortran -lm";
   }
   if ( $abi eq "-o32" ){
     $libs = "-L/usr/lib -lF77 -lI77 -lU77 -lisam -lm";
@@ -217,10 +333,19 @@ $F77config{Freebsd}{DEFAULT}     = 'F77';
 ### VMS ###
 
 $F77config{VMS}{Fortran}{Trail_} = 0;
-$F77config{VMS}{Fortran}{Link}   = '';
+$F77config{VMS}{Fortran}{Link}   = ' ';         # <---need this space!
 $F77config{VMS}{DEFAULT}     = 'Fortran';
 $F77config{VMS}{Fortran}{Compiler} = 'Fortran';
 
+### Darwin (Mac OS X) ###
+
+$F77config{Darwin}{GFortran}{Trail_} = 1;
+$F77config{Darwin}{GFortran}{Cflags} = ' ';        # <---need this space!
+$F77config{Darwin}{GFortran}{Link}   = '-L/usr/local/lib -lgfortran';    
+$F77config{Darwin}{GFortran}{Compiler} = 'gfortran';
+
+$F77config{Darwin}{DEFAULT}     = 'G77';
+
 ############ End of database is here ############ 
 
 =head1 SYNOPSIS
@@ -247,9 +372,11 @@ sub get; # See below
 # a lot of the answers depend on a lot of the guesswork.
 
 sub import {
+   no warnings; # Shuts up complaints on Win32 when running PDL tests
    $Pkg    = shift;
    my $system   = ucfirst(shift);  # Set package variables
    my $compiler = ucfirst(shift);
+   $Runtime = "-LSNAFU -lwontwork";
 
    # Guesses if system/compiler not specified.
 
@@ -257,21 +384,27 @@ sub import {
    $system = 'Cygwin' if $system =~ /Cygwin/;
    $compiler = get $F77config{$system}{DEFAULT} unless $compiler;
 
-   print "$Pkg: Using system=$system compiler=$compiler\n";
-   
+   print "$Pkg: Using system=$system compiler=" .
+       (defined $compiler ? $compiler : "<undefined>") . "\n";
+
    if (defined($ENV{F77LIBS})) {
       print "Overriding Fortran libs from value of enviroment variable F77LIBS = $ENV{F77LIBS}\n";
       $Runtime = $ENV{F77LIBS};
    }
    else {
-      
+
      # Try this combination
 
-     if (defined( $F77config{$system} )){
+     if ( defined( $compiler ) and defined( $F77config{$system} )){
      	my $flibs = get ($F77config{$system}{$compiler}{Link});
         if ($flibs ne "") {
-     	   $Runtime = $flibs . gcclibs();
+     	   $Runtime = $flibs;# . gcclibs();
+	   # We don't want to append gcc libs if we are using
+	   # non gnu compilers. Initially, explicitly check for
+	   # use of sun compiler
+	   $Runtime .= gcclibs($flibs) unless $flibs =~ /sunmath/;
 	   $Runtime =~ s|L([a-z,A-Z]):|L//$1|g if $^O =~ /cygwin/i;
+           $Runtime = ' ' if $^O eq 'VMS';  # <-- need this space!
 	   print "Runtime: $Runtime\n";
            $ok = 1;
      	   $ok = validate_libs($Runtime) if $flibs ne "";
@@ -282,7 +415,7 @@ sub import {
 
      # If it doesn't work try Generic + GNU77
 
-     unless (defined($Runtime) && $ok) {
+      unless (("$Runtime" ne "-LSNAFU -lwontwork") && $ok) {
      	print <<"EOD";
 $Pkg: Unable to guess and/or validate system/compiler configuration
 $Pkg: Will try system=Generic Compiler=G77
@@ -290,18 +423,18 @@ EOD
     	 $system   = "Generic";
     	 $compiler = "G77";
     	 my $flibs = get ($F77config{$system}{$compiler}{Link});
-    	 $Runtime =  $flibs. gcclibs();
+    	 $Runtime =  $flibs . gcclibs($flibs);
     	 $ok = validate_libs($Runtime) if $flibs ne "";
     	 print "$Pkg: Well that didn't appear to validate. Well I will try it anyway.\n"
     	      unless $Runtime && $ok;
        }
- 
+
       $RuntimeOK = $ok;
-      
+
    } # Not overriding   
 
    # Now get the misc info for the methods.
-      
+
    if (defined( $F77config{$system}{$compiler}{Trail_} )){
       $Trail_  = get $F77config{$system}{$compiler}{Trail_};  
    }
@@ -313,9 +446,9 @@ $Pkg: F77 names have trailing underscores.
 EOD
       $Trail_ = 1;
    }
-  
+
    if (defined( $F77config{$system}{$compiler}{Compiler} )) {
-	$Compiler = $F77config{$system}{$compiler}{Compiler};
+	$Compiler = get $F77config{$system}{$compiler}{Compiler};
    } else {
 	print << "EOD";
 $Pkg: There does not appear to be any configuration info about
@@ -326,7 +459,7 @@ EOD
 print "$Pkg: Compiler: $Compiler\n";
 
    if (defined( $F77config{$system}{$compiler}{Cflags} )) {
-	$Cflags = $F77config{$system}{$compiler}{Cflags};
+	$Cflags = get $F77config{$system}{$compiler}{Cflags} ;
    } else {
 	print << "EOD";
 $Pkg: There does not appear to be any configuration info about
@@ -337,7 +470,7 @@ EOD
    }
 
    print "$Pkg: Cflags: $Cflags\n";
-   
+
 } # End of import ()
 
 =head1 METHODS
@@ -399,7 +532,6 @@ sub any_exists {
 
 # Find highest version number of SCN.N(.N) directories
 # (Nasty SunOS/Solaris naming scheme for F77 libs]
-
 sub find_highest_SC {
     print "$Pkg: Scanning for $_[0]\n";
     my @glob = glob(shift);
@@ -413,10 +545,11 @@ sub find_highest_SC {
            $n{$_} = $1 *100 + $2 * 10 + $3;
        }
     }
-    my @sorted_dirs = sort {$n{$a} <=> $n{$b}} @glob;
+    my @sorted_dirs = grep {-f "$_/libF77.a"} sort {$n{$a} <=> $n{$b}} @glob;
+
     return pop @sorted_dirs; # Highest N
 }
-     
+
 # Validate a string of form "-Ldir -lfoo -lbar"
 
 sub validate_libs {
@@ -452,8 +585,10 @@ sub validate_libs {
 
 
 sub testcompiler {
- 
-    my $file = "/tmp/testf77$$";
+
+    my $file = File::Spec->tmpdir . "/testf77$$";
+    $file =~ s/\\/\//g; # For Win32
+
     my $ret;
     open(OUT,">$file.f");
     print OUT "      print *, 'Hello World'\n";
@@ -478,6 +613,7 @@ sub testcompiler {
 # Return gcc libs (e.g. -L/usr/local/lib/gcc-lib/sparc-sun-sunos4.1.3_U1/2.7.0 -lgcc)
 
 sub gcclibs {
+   my $flibs = shift; # Fortran libs
    my $isgcc = $Config{'cc'} eq 'gcc';
    if (!$isgcc && $^O ne 'VMS') {
       print "Checking for gcc in disguise:\n";
@@ -491,7 +627,7 @@ sub gcclibs {
       }
       print $string;
    }
-   if ($isgcc) {
+   if ($isgcc or ($flibs =~ /-lg2c/) or ($flibs =~ /-lf2c/) ) {
        $gccdir = `gcc -print-libgcc-file-name`; chomp $gccdir;
        $gccdir =~ s/\/libgcc.a//;
        return " -L$gccdir -lgcc";   
@@ -504,7 +640,12 @@ sub gcclibs {
 
 sub find_in_path {
    my @names = @_;
-   my @path = split(":",$ENV{PATH});
+   my @path;
+   if($^O =~ /mswin32/i) {
+     for(@names) { $_ .= '.exe'}
+     @path = split(";", $ENV{PATH});
+     }
+   else {@path = split(":",$ENV{PATH})}
    my ($name,$dir);
    for $name (@names) {
       for $dir (@path) {
@@ -515,7 +656,8 @@ sub find_in_path {
       }
    }
    return '' if $^O eq 'VMS';
-   die "Unable to find a fortran compiler using names: ".join(" ",@names);
+   return undef;
+#   die "Unable to find a fortran compiler using names: ".join(" ",@names);
 }
 
 
